/*
   _===_ _           _                  ______ _     _             _ _ _
  |  ___| |         | |                 | ___ (_)   | |           (_) | |
  | |__ | | ___  ___| |_ _ __ ___ ______| |_/ /_  __| | ___  _   _ _| | | ___ _   _ _ __
  |  __|| |/ _ \/ __| __| '__/ _ \______| ___ \ |/ _` |/ _ \| | | | | | |/ _ \ | | | '__|
  | |___| |  __/ (__| |_| | | (_) |     | |_/ / | (_| | (_) | |_| | | | |  __/ |_| | |
  \____/|_|\___|\___|\__|_|  \___/      \____/|_|\__,_|\___/ \__,_|_|_|_|\___|\__,_|_|

  Bidouilleur PIC12F683 - exemple de Contr[ole de Servomoteur 
  lectro-Bidouilleur, Mars 2018


	Compil pour: PIC12F683
	Compilateur: Microchip XC8
	IDE: Microchip MPLABX
*/


#include <xc.h>
#include <pic12f683.h>
#define  test_bit(var, bit)   ((var) & (1 <<(bit)))
#define  set_bit(var, bit)    ((var) |= (1 << (bit)))
#define  clear_bit(var, bit)    ((var) &= ~(1 << (bit)))
#define _XTAL_FREQ                  125000

// GLOBALS
unsigned char           i;      // Variables compteur diverses
unsigned char           j;


// CONFIGURATION BYTES
#pragma config FOSC = INTOSCIO  // Oscillator Selection bits (INTOSCIO oscillator: I/O function on RA4/OSC2/CLKOUT pin, I/O function on RA5/OSC1/CLKIN)
#pragma config WDTE = OFF       // Watchdog Timer Enable bit (WDT disabled)
#pragma config PWRTE = OFF      // Power-up Timer Enable bit (PWRT disabled)
#pragma config MCLRE = OFF      // MCLR Pin Function Select bit (MCLR pin function is digital input, MCLR internally tied to VDD)
#pragma config CP = OFF         // Code Protection bit (Program memory code protection is disabled)
#pragma config CPD = OFF        // Data Code Protection bit (Data memory code protection is disabled)
#pragma config BOREN = OFF      // Brown Out Detect (BOR disabled)
#pragma config IESO = OFF       // Internal External Switchover bit (Internal External Switchover mode is disabled)
#pragma config FCMEN = OFF      // Fail-Safe Clock Monitor Enabled bit (Fail-Safe Clock Monitor is disabled)

// CONSTANTS AND MASKS DEFINITION

	//Other definitions
//#define LED 				2       // The bit position on GPIO for the LED

void delay_ms(int delay) {
    for (i = 0; i < delay; i++) __delay_ms(1);
}

void main(void) 
{  	// Initialization stage
    //Set CPU to 125KHz Here.
    // Initialization stage
    INTCON = 0b00000000; // disable interrupts
    IOC = 0b00000000; // disable GPIO interrupts on change
    PIE1 = 0b00000000; // disable various interrupts
    OPTION_REG = 0b00001111; // weak pullups global enable,
    WPU = 0b00110111; // individual weak pullups enabled
    OSCCON = 0b00010001; // select internal oscillator and set it to 125Khz
    T1CON = 0b10110111; // enable timer1 and set prescaler div. by 1
    T2CON = 0b00000101; // enable timer2 and set prescaler div. by 4 and postscaler div. by 1
    CCP1CON = 0b00001100; // Set PWM mode, active high
    ADCON0 = 0b00000000; // disable A/D converter
    ANSEL = 0b00000000; // Disable analog assignment to pins
    CMCON0 = 0b00010111; // Disable comparator, use internal reference, invert output.
    CMCON1 = 0b00000000; // comparator gating timer 1 asynchronously
    VRCON = 0b00100001; // Disable internal comparator Vref, use low range, set Vref to 0.15V
    WDTCON = 0b00010100; // Assign the right prescaler value to the watchdog timer. Keep WTD off for now.
    GPIO   = 0b00110000; // Set output pins initially before enabling them.
    TRISIO = 0b00111011; // GPIO first disabled, but assigned as follows:
									  //	5-Available
									  //	4-Available
									  //	3-Input only. 
									  //	2-CCP1 output. 
									  //	1-Available
									  //	0-Available

    PR2 = 155;          // Priode du PWM (20 ms)
    while (1)
    {
        for (j=31;j<=62;j++)    // j: balayage (de 5%  10%) du rapport cyclique du PWM
        {    
            CCPR1L = j>>2;                                              // criture du nouveau rapport cyclique
            CCP1CON = (CCP1CON & 0b11001111) | ((j & 0b00000011)<<4);   //              "
            delay_ms(50);           // Petite pause pour ralentir le va-et-vient
        }
        for (j=62;j>=31;j--)    // j: balayage  (de 10%  5%) du rapport cyclique du PWM
        {    
            CCPR1L = j>>2;                                              // criture du nouveau rapport cyclique
            CCP1CON = (CCP1CON & 0b11001111) | ((j & 0b00000011)<<4);   //              "
            delay_ms(50);           // Petite pause pour ralentir le va-et-vient
        }
    }
}